use sea_orm_migration::prelude::*;

use crate::{m20230204_011034_create_posts::Post, m20230207_115847_create_images::Image};

#[derive(DeriveMigrationName)]
pub struct Migration;

#[async_trait::async_trait]
impl MigrationTrait for Migration {
    async fn up(&self, manager: &SchemaManager) -> Result<(), DbErr> {
        manager
            .alter_table(
                Table::alter()
                    .table(Post::Table)
                    .add_column(ColumnDef::new(Alias::new("featured_image")).string())
                    .add_foreign_key(&TableForeignKey::new()
                        .name("fk-featured_image-image")
                        .from_tbl(Post::Table)
                        .from_col(Alias::new("featured_image"))
                        .to_tbl(Image::Table)
                        .to_col(Image::Id)
                        .on_update(ForeignKeyAction::Cascade)
                        .on_delete(ForeignKeyAction::SetNull)
                    )
                    .to_owned(),
            )
            .await
    }

    async fn down(&self, manager: &SchemaManager) -> Result<(), DbErr> {
        manager
            .alter_table(
                    Table::alter()
                        .table(Post::Table)
                        .drop_column(Alias::new("featured_image"))
                        .drop_foreign_key(Alias::new("fk-featured_image-image"))
                .to_owned()
            )
            .await
    }
}