use askama::Template;
use askama_axum::IntoResponse;
use axum::extract::State;
use axum::http::StatusCode;

use crate::AppState;

use crate::entities::{prelude::*, *};
use sea_orm::*;

#[derive(Template)]
#[template(path = "posts_listing.html")]
pub(crate) struct PageTemplate {
    menu: Vec<menu_entry::Model>,
    posts: Vec<post::Model>,
    year: String,
    settings: settings::Model,
}

pub(crate) async fn posts(state: State<AppState>) -> impl IntoResponse {
    let menu: Vec<menu_entry::Model> = match Menu::find()
        .filter(menu::Column::Name.eq("primary"))
        .one(&state.db_conn)
        .await
        .expect("Failed to get primary menu! HALTING")
    {
        Some(menu) => menu,
        None => panic!("Missing \"primary\" menu in database!"),
    }
    .find_related(MenuEntry)
    .order_by_asc(menu_entry::Column::Order)
    .all(&state.db_conn)
    .await
    .expect("Failed to get primary menu items!");

    let settings: settings::Model = match Settings::find()
        .filter(settings::Column::Id.eq("current"))
        .one(&state.db_conn)
        .await
        .expect("Failed to get site settings!")
    {
        Some(settings) => settings,
        None => panic!("Missing \"current\" settings in database!"),
    };

    let posts: Vec<post::Model> = Post::find()
        .filter(post::Column::Draft.eq(false))
        .order_by_desc(post::Column::Published)
        .all(&state.db_conn)
        .await
        .expect("Failed to load posts!");

    (
        StatusCode::OK,
        PageTemplate {
            menu,
            posts,
            year: "2023".to_owned(),
            settings,
        },
    )
}
