use crate::entities::{prelude::*, *};
use crate::AppState;
use sea_orm::*;

use axum::extract::State;
use axum::http::{header, StatusCode};
use axum::response::IntoResponse;
use rss::{ChannelBuilder, ItemBuilder};

pub(crate) async fn build_rss(state: State<AppState>) -> impl IntoResponse {
    let settings: settings::Model = match Settings::find()
        .filter(settings::Column::Id.eq("current"))
        .one(&state.db_conn)
        .await
        .expect("Failed to get site settings!")
    {
        Some(settings) => settings,
        None => panic!("Missing \"current\" settings in database!"),
    };

    let posts: Vec<post::Model> = Post::find()
        .filter(post::Column::Draft.eq(false))
        .order_by_desc(post::Column::Published)
        .all(&state.db_conn)
        .await
        .expect("Failed to load posts!");

    let channel = ChannelBuilder::default()
        .generator("Kyanite".to_owned())
        .title(settings.site_name)
        .link(&settings.site_url)
        .description(settings.site_description.unwrap_or(settings.site_header))
        .items(
            posts
                .into_iter()
                .map(|f| {
                    ItemBuilder::default()
                        .guid(rss::Guid {
                            value: f.id,
                            permalink: false,
                        })
                        .title(f.title)
                        .description(f.summary)
                        .pub_date(f.published.format("%a, %d %b %Y %H:%M:%S GMT").to_string())
                        .link(format!("{}/post/{}", settings.site_url, f.slug))
                        .build()
                })
                .collect::<Vec<rss::Item>>(),
        )
        .build();

    (
        StatusCode::OK,
        [(header::CONTENT_TYPE, "application/rss+xml")],
        channel.to_string(),
    )
}
